%bcond test 0

Name:           z3
Version:        4.12.6
Release:    1%{?dist}
Summary:        Satisfiability Modulo Theories (SMT) solver

License:        MIT
URL:            https://github.com/Z3Prover/z3/wiki
VCS:            https://github.com/Z3Prover/z3
Source0:        %{vcs}/archive/%{name}-%{version}.tar.gz
Patch0:         %{name}-ocaml.patch

BuildRequires:  cmake
BuildRequires:  doxygen
BuildRequires:  gcc-c++
BuildRequires:  gmp-devel
BuildRequires:  graphviz
BuildRequires:  help2man
%ifarch %{java_arches}
BuildRequires:  java-devel
BuildRequires:  javapackages-tools
%endif
BuildRequires:  make
BuildRequires:  ninja-build
%ifnarch %{ix86}
BuildRequires:  ocaml
BuildRequires:  ocaml-findlib
BuildRequires:  ocaml-ocamldoc
BuildRequires:  ocaml-zarith-devel
%endif
BuildRequires:  python3-devel
BuildRequires:  %{py3_dist setuptools}

%description
Z3 is a satisfiability modulo theories (SMT) solver; given a set of
constraints with variables, it reports a set of values for those
variables that would meet the constraints.  The Z3 input format is an
extension of the one defined by the SMT-LIB 2.0 standard.  Z3 supports
arithmetic, fixed-size bit-vectors, extensional arrays, datatypes,
uninterpreted functions, and quantifiers.

%package libs
Summary:        Library for applications that use z3 functionality

%ifnarch %{java_arches}
%endif

%description libs
Library for applications that use z3 functionality.

%package devel
Summary:        Header files for build applications that use z3
Requires:       z3-libs = %{version}-%{release}

%description devel
Header files for build applications that use z3.

%package doc
License:        MIT AND GPL-3.0-or-later WITH Bison-exception-2.2 AND GPL-1.0-or-later
Summary:        API documentation for Z3

%description doc
API documentation for Z3.

%ifarch %{java_arches}
%package -n java-z3
Summary:        Java interface to z3
Requires:       z3-libs = %{version}-%{release}
Requires:       java
Requires:       javapackages-tools

%description -n java-z3
Java interface to z3.
%endif

%ifnarch %{ix86}
%package -n ocaml-z3
Summary:        Ocaml interface to z3
Requires:       z3-libs = %{version}-%{release}

%description -n ocaml-z3
Ocaml interface to z3.

%package -n ocaml-z3-devel
Summary:        Files for building ocaml applications that use z3
Requires:       ocaml-z3 = %{version}-%{release}
Requires:       ocaml-zarith-devel

%description -n ocaml-z3-devel
Files for building ocaml applications that use z3.
%endif

%package -n python3-z3
Summary:        Python 3 interface to z3
BuildArch:      noarch
Requires:       z3-libs = %{version}-%{release}

%description -n python3-z3
Python 3 interface to z3.

%prep
%autosetup -N -n %{name}-%{name}-%{version}
%ifnarch %{ocaml_native_compiler}
%patch -P0 -p1
%endif
%autopatch -m 1 -p1

sed \
  -e 's/@$(CXX)/$(CXX)/' \
  -e '/O3/d' \
  -e "s/\(['\"]\)cp\([^[:alnum:]]\)/\1cp -p\2/" \
  -e "s/\(SLIBEXTRAFLAGS = '\)'/\1-Wl,--no-whole-archive'/" \
  -e '/SLIBFLAGS/s|-shared|& %{build_ldflags} -Wl,--whole-archive|' \
  -e 's/\(libz3$(SO_EXT)\)\(\\n\)/\1 -Wl,--no-whole-archive\2/' \
  -e "s/OCAML_FLAGS = ''/OCAML_FLAGS = '-g'/" \
  -i scripts/mk_util.py

majver=$(cut -d. -f-2 <<< %{version})
sed -e '/libz3java/s,\(System\.load\)Library("\(.*\)"),\1("%{_libdir}/z3/\2.so"),' \
    -e "s/'so'/'so.$majver'/" \
    -i scripts/update_api.py

sed -i '/HTML_TIMESTAMP/s/YES/NO/' doc/z3api.cfg.in doc/z3code.dox

%build
export LANG=C.UTF-8
export PYTHON=%{python3}

%cmake -G Ninja \
  -DCMAKE_INSTALL_INCLUDEDIR=%{_includedir}/z3 \
  -DCMAKE_JAVA_COMPILE_FLAGS="-source;1.8;-target;1.8" \
  -DZ3_BUILD_DOCUMENTATION:BOOL=ON \
%ifarch %{java_arches}
  -DZ3_BUILD_JAVA_BINDINGS:BOOL=ON \
%endif
  -DZ3_BUILD_PYTHON_BINDINGS:BOOL=ON \
  -DZ3_INCLUDE_GIT_HASH:BOOL=OFF \
  -DZ3_INCLUDE_GIT_DESCRIBE:BOOL=OFF \
  -DZ3_USE_LIB_GMP:BOOL=ON

%cmake_build

%ifnarch %{ix86}
./configure -p %{_prefix} --gmp --ml

cp -dp %{_vpath_builddir}/libz3.so* build

sed -i '/^api/s/ libz3\$(SO_EXT)//g' build/Makefile

%make_build -C build ml
%endif

%install
export LANG="C.UTF-8"

%cmake_install

%ifarch %{java_arches}
mkdir -p %{buildroot}%{_libdir}/z3
mkdir -p %{buildroot}%{_jnidir}
mv %{buildroot}%{_javadir}/*.jar %{buildroot}%{_jnidir}
ln -s %{_jnidir}/com.microsoft.z3.jar %{buildroot}%{_libdir}/z3
mv %{buildroot}%{_libdir}/libz3java.so %{buildroot}%{_libdir}/z3
%endif

%ifnarch %{ix86}
cd build/api/ml
mkdir -p %{buildroot}%{ocamldir}/Z3
%ifarch %{ocaml_native_compiler}
cp -p *.cmx{,a,s} %{buildroot}%{ocamldir}/Z3
%endif
cp -p META *.{a,cma,cmi,mli} %{buildroot}%{ocamldir}/Z3
mkdir -p %{buildroot}%{ocamldir}/stublibs
cp -p *.so %{buildroot}%{ocamldir}/stublibs
cd -
%endif

rm -rf %{buildroot}%{_docdir}/Z3

mkdir -p %{buildroot}%{_mandir}/man1
help2man -N -o %{buildroot}%{_mandir}/man1/z3.1 %{_vpath_builddir}/z3

sed -i 's,//usr,,' %{buildroot}%{_libdir}/pkgconfig/z3.pc

%if %{with test}
%check
export LANG="C.UTF-8"
cd build
make test-z3
./test-z3 /a
cd -
%endif

%files
%doc README.md RELEASE_NOTES.md
%{_bindir}/z3
%{_mandir}/man1/z3.1*

%files libs
%license LICENSE.txt
%{_libdir}/libz3.so.4.12*

%files devel
%{_includedir}/z3/
%{_libdir}/libz3.so
%{_libdir}/cmake/z3/
%{_libdir}/pkgconfig/z3.pc

%files doc
%doc %{_vpath_builddir}/doc/api/html examples
%license LICENSE.txt

%ifarch %{java_arches}
%files -n java-z3
%{_libdir}/z3/
%{_jnidir}/com.microsoft.z3*jar
%endif

%ifnarch %{ix86}
%files -n ocaml-z3
%dir %{ocamldir}/Z3/
%{ocamldir}/Z3/META
%{ocamldir}/Z3/*.cma
%{ocamldir}/Z3/*.cmi
%ifarch %{ocaml_native_compiler}
%{ocamldir}/Z3/*.cmxs
%endif
%{ocamldir}/stublibs/*.so

%files -n ocaml-z3-devel
%{ocamldir}/Z3/*.a
%ifarch %{ocaml_native_compiler}
%{ocamldir}/Z3/*.cmx
%{ocamldir}/Z3/*.cmxa
%endif
%{ocamldir}/Z3/*.mli
%endif

%files -n python3-z3
%{python3_sitelib}/z3/

%changelog
* Wed Aug 07 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 4.12.6-1
- initial build
